"use strict";
/*
 * @microeinhundert/radonis-server
 *
 * (c) Leon Seipp <l.seipp@microeinhundert.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const node_path_1 = require("node:path");
/**
 * Returns absolute path to the stub relative
 * from the templates directory
 */
function getStub(...relativePaths) {
    return (0, node_path_1.join)(__dirname, 'templates', ...relativePaths);
}
/**
 * Creates the entry file
 */
async function createEntryFile(app, sink, projectRoot) {
    const resourcesDir = app.directoriesMap.get('resources') || 'resources';
    const entryFile = app.resourcesPath('entry.client.ts');
    const entryFileTemplate = new sink.files.MustacheFile(projectRoot, entryFile, getStub('entry.client.txt'));
    if (entryFileTemplate.exists()) {
        sink.logger.action('create').skipped(`${resourcesDir}/entry.client.ts`);
    }
    else {
        entryFileTemplate.apply({}).commit();
        sink.logger.action('create').succeeded(`${resourcesDir}/entry.client.ts`);
    }
}
/**
 * Installs packages
 */
async function installPackages(sink, projectRoot) {
    const packageJsonFile = new sink.files.PackageJsonFile(projectRoot);
    /**
     * Dependencies
     */
    packageJsonFile.install('@microeinhundert/radonis-unocss', '^2.1.5', false);
    packageJsonFile.install('@adonisjs/i18n', '^1.5.0', false);
    packageJsonFile.install('@adonisjs/session', '^6.4.0', false);
    packageJsonFile.install('react', '^18.2.0', false);
    packageJsonFile.install('react-dom', '^18.2.0', false);
    /**
     * Dev Dependencies
     */
    packageJsonFile.install('@types/react', '^18.0.0', true);
    packageJsonFile.install('@types/react-dom', '^18.0.0', true);
    packageJsonFile.install('concurrently', undefined, true);
    const spinner = sink.logger.await('Installing packages');
    try {
        await packageJsonFile.commitAsync();
        spinner.update('Packages installed');
    }
    catch (error) {
        spinner.update('Unable to install packages');
        sink.logger.fatal(error);
    }
    spinner.stop();
}
/**
 * Adds scripts
 */
async function addScripts(sink, projectRoot) {
    const packageJsonFile = new sink.files.PackageJsonFile(projectRoot);
    packageJsonFile.setScript('dev', "concurrently 'npm:dev:*'");
    packageJsonFile.setScript('dev:client', 'node ace build:client --watch');
    packageJsonFile.setScript('dev:server', 'node ace serve --watch');
    packageJsonFile.setScript('build', 'npm run build:server && npm run build:client');
    packageJsonFile.setScript('build:client', 'node ace build:client --production');
    packageJsonFile.setScript('build:server', 'node ace build --production');
    await packageJsonFile.commitAsync();
}
/**
 * Modifies tsconfig
 */
function modifyTsconfig(sink, projectRoot) {
    const tsConfigFile = new sink.files.JsonFile(projectRoot, 'tsconfig.json');
    tsConfigFile.set('compilerOptions.jsx', 'react-jsx');
    tsConfigFile.commit();
}
/**
 * Instructions to be executed when setting up the package
 */
async function instructions(projectRoot, app, sink) {
    createEntryFile(app, sink, projectRoot);
    try {
        await installPackages(sink, projectRoot);
        await addScripts(sink, projectRoot);
        modifyTsconfig(sink, projectRoot);
    }
    catch (error) {
        sink.logger.fatal(error);
    }
}
exports.default = instructions;
//# sourceMappingURL=instructions.js.map