"use strict";
/*
 * @microeinhundert/radonis-server
 *
 * (c) Leon Seipp <l.seipp@microeinhundert.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.HeadManager = void 0;
const radonis_shared_1 = require("@microeinhundert/radonis-shared");
const build_title_1 = require("../utils/build_title");
/**
 * Service for managing head data
 */
class HeadManager {
    /**
     * The singleton instance
     */
    static instance;
    /**
     * Get the singleton instance
     */
    static getSingletonInstance(...args) {
        return (HeadManager.instance = HeadManager.instance ?? new HeadManager(...args));
    }
    /**
     * The Radonis config
     */
    #config;
    /**
     * The title
     */
    #title;
    /**
     * The meta
     */
    #meta;
    /**
     * The tags
     */
    #tags;
    constructor(config) {
        this.#config = config;
        this.#setDefaults();
    }
    /**
     * Set the title
     */
    setTitle(title) {
        const { prefix, suffix, separator } = this.#config.head.title;
        this.#title = (0, build_title_1.buildTitle)(title, prefix, suffix, separator);
    }
    /**
     * Get the title markup
     */
    #getTitleMarkup() {
        return `<title>${this.#title}</title>`;
    }
    /**
     * Add meta
     */
    addMeta(meta) {
        this.#meta = { ...this.#meta, ...meta };
    }
    /**
     * Get the meta markup
     */
    #getMetaMarkup() {
        return (0, radonis_shared_1.nonNull)(Object.entries(this.#meta).map(([name, value]) => {
            if (value === null || value === undefined) {
                return null;
            }
            const isOpenGraphTag = /^(og|music|video|article|book|profile|fb):.+$/.test(name);
            const isCharsetTag = ['charset', 'charSet'].includes(name);
            if (isCharsetTag) {
                return `<meta ${(0, radonis_shared_1.stringifyAttributes)({ charset: value })} />`;
            }
            return `<meta ${(0, radonis_shared_1.stringifyAttributes)({ value, [isOpenGraphTag ? 'property' : 'name']: name })} />`;
        })).join('\n');
    }
    /**
     * Add tags
     */
    addTags(tags) {
        this.#tags = [...this.#tags, ...tags];
    }
    /**
     * Get the tags markup
     */
    #getTagsMarkup() {
        return this.#tags
            .map(({ name, content, attributes }) => {
            return `<${name}${attributes ? ` ${(0, radonis_shared_1.stringifyAttributes)(attributes)}` : ''}>${content}</${name}>`;
        })
            .join('\n');
    }
    /**
     * Get the markup, including the opening and closing head tags
     */
    getMarkup() {
        return ['<head>', this.#getTitleMarkup(), this.#getMetaMarkup(), this.#getTagsMarkup(), '</head>'].join('\n');
    }
    /**
     * Reset for a new request
     */
    reset() {
        this.#setDefaults();
    }
    /**
     * Set the defaults
     */
    #setDefaults() {
        this.setTitle(this.#config.head.title.default);
        this.#meta = this.#config.head.defaultMeta;
        this.#tags = [];
    }
}
exports.HeadManager = HeadManager;
//# sourceMappingURL=main.js.map