"use strict";
/*
 * @microeinhundert/radonis-server
 *
 * (c) Leon Seipp <l.seipp@microeinhundert.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.HydrationManager = void 0;
/**
 * Service for managing hydration requirements
 */
class HydrationManager {
    /**
     * The singleton instance
     */
    static instance;
    /**
     * Get the singleton instance
     */
    static getSingletonInstance(...args) {
        return (HydrationManager.instance = HydrationManager.instance ?? new HydrationManager(...args));
    }
    /**
     * The flash messages
     */
    #flashMessages;
    #requiredFlashMessages;
    /**
     * The messages
     */
    #messages;
    #requiredMessages;
    /**
     * The routes
     */
    #routes;
    #requiredRoutes;
    constructor() {
        this.#setDefaults();
    }
    /**
     * The flash messages
     */
    get flashMessages() {
        return this.#flashMessages;
    }
    get requiredFlashMessages() {
        const flashMessages = {};
        for (const identifier of this.#requiredFlashMessages) {
            if (identifier in this.#flashMessages) {
                flashMessages[identifier] = this.#flashMessages[identifier];
            }
        }
        return flashMessages;
    }
    /**
     * Set the flash messages
     */
    setFlashMessages(flashMessages) {
        this.#flashMessages = flashMessages;
        return this;
    }
    /**
     * Require a flash message
     */
    requireFlashMessage(identifier) {
        if (identifier in this.#flashMessages) {
            this.#requiredFlashMessages.add(identifier);
        }
        return this;
    }
    /**
     * Require all flash messages
     */
    requireAllFlashMessages() {
        this.#requiredFlashMessages = new Set(Object.keys(this.#flashMessages));
        return this;
    }
    /**
     * The messages
     */
    get messages() {
        return this.#messages;
    }
    get requiredMessages() {
        const messages = {};
        for (const identifier of this.#requiredMessages) {
            if (identifier in this.#messages) {
                messages[identifier] = this.#messages[identifier];
            }
        }
        return messages;
    }
    /**
     * Set the messages
     */
    setMessages(messages) {
        this.#messages = messages;
        return this;
    }
    /**
     * Require a message
     */
    requireMessage(identifier) {
        if (identifier in this.#messages) {
            this.#requiredMessages.add(identifier);
        }
        return this;
    }
    /**
     * Require all messages
     */
    requireAllMessages() {
        this.#requiredMessages = new Set(Object.keys(this.#messages));
        return this;
    }
    /**
     * The routes
     */
    get routes() {
        return this.#routes;
    }
    get requiredRoutes() {
        const routes = {};
        for (const identifier of this.#requiredRoutes) {
            if (identifier in this.#routes) {
                routes[identifier] = this.#routes[identifier];
            }
        }
        return routes;
    }
    /**
     * Set the routes
     */
    setRoutes(routes) {
        this.#routes = routes;
        return this;
    }
    /**
     * Require a route
     */
    requireRoute(identifier) {
        if (identifier in this.#routes) {
            this.#requiredRoutes.add(identifier);
        }
        return this;
    }
    /**
     * Require all routes
     */
    requireAllRoutes() {
        this.#requiredRoutes = new Set(Object.keys(this.#routes));
        return this;
    }
    /**
     * Require flash messages, messages and routes by tokens
     */
    requireTokens(tokens) {
        for (const token of tokens) {
            this.requireFlashMessage(token);
            this.requireMessage(token);
            this.requireRoute(token);
        }
        return this;
    }
    /**
     * Reset for a new request
     */
    reset() {
        this.#setDefaults();
    }
    /**
     * Set the defaults
     */
    #setDefaults() {
        this.#flashMessages = {};
        this.#requiredFlashMessages = new Set();
        this.#messages = {};
        this.#requiredMessages = new Set();
        this.#routes = {};
        this.#requiredRoutes = new Set();
    }
}
exports.HydrationManager = HydrationManager;
//# sourceMappingURL=main.js.map