"use strict";
/*
 * @microeinhundert/radonis-server
 *
 * (c) Leon Seipp <l.seipp@microeinhundert.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.ManifestManager = void 0;
const tslib_1 = require("tslib");
const radonis_shared_1 = require("@microeinhundert/radonis-shared");
const superjson_1 = tslib_1.__importDefault(require("superjson"));
const exceptions_1 = require("../exceptions");
const constants_1 = require("./constants");
/**
 * Service for managing the manifest
 */
class ManifestManager {
    /**
     * The singleton instance
     */
    static instance;
    /**
     * Get the singleton instance
     */
    static getSingletonInstance(...args) {
        return (ManifestManager.instance = ManifestManager.instance ?? new ManifestManager(...args));
    }
    /**
     * The Radonis config
     */
    #config;
    /**
     * The HydrationManager instance
     */
    #hydrationManager;
    /**
     * The hydration
     */
    #hydration;
    /**
     * The globals
     */
    #globals;
    /**
     * The locale
     */
    #locale;
    /**
     * The current route
     */
    #route;
    constructor(config, application) {
        this.#config = config;
        this.#hydrationManager = application.container.resolveBinding('Microeinhundert/Radonis/HydrationManager');
        this.#setDefaults();
    }
    /**
     * The server manifest
     */
    get #serverManifest() {
        return {
            hydration: this.#hydration,
            globals: this.#globals,
            locale: this.#locale,
            route: this.#route,
            flashMessages: this.flashMessages,
            messages: this.messages,
            routes: this.routes,
        };
    }
    /**
     * The client manifest
     */
    get #clientManifest() {
        return {
            hydration: this.#hydration,
            globals: this.#globals,
            locale: this.#locale,
            route: this.#route,
            flashMessages: this.#config.client.limitManifest ? this.flashMessagesRequiredForHydration : this.flashMessages,
            messages: this.#config.client.limitManifest ? this.messagesRequiredForHydration : this.messages,
            routes: this.#config.client.limitManifest ? this.routesRequiredForHydration : this.routes,
        };
    }
    /**
     * The flash messages
     */
    get flashMessages() {
        return this.#hydrationManager.flashMessages;
    }
    get flashMessagesRequiredForHydration() {
        return this.#hydrationManager.requiredFlashMessages;
    }
    /**
     * Set the flash messages
     */
    setFlashMessages(flashMessages) {
        this.#hydrationManager.setFlashMessages(flashMessages);
        return this;
    }
    /**
     * The messages
     */
    get messages() {
        return this.#hydrationManager.messages;
    }
    get messagesRequiredForHydration() {
        return this.#hydrationManager.requiredMessages;
    }
    /**
     * Set the messages
     */
    setMessages(messages) {
        this.#hydrationManager.setMessages(messages);
        return this;
    }
    /**
     * The routes
     */
    get routes() {
        return this.#hydrationManager.routes;
    }
    get routesRequiredForHydration() {
        return this.#hydrationManager.requiredRoutes;
    }
    /**
     * Set the routes
     */
    setRoutes(routes) {
        this.#hydrationManager.setRoutes(routes);
        return this;
    }
    /**
     * The hydration
     */
    get hydration() {
        return this.#hydration;
    }
    /**
     * Register hydration
     */
    registerHydration(hydrationRootId, islandIdentifier, props) {
        this.#hydration = {
            ...this.#hydration,
            [hydrationRootId]: { islandIdentifier, props },
        };
        return this;
    }
    /**
     * The globals
     */
    get globals() {
        return this.#globals;
    }
    /**
     * Add globals
     */
    addGlobals(globals) {
        this.#globals = { ...this.#globals, ...globals };
        return this;
    }
    /**
     * The locale
     */
    get locale() {
        return this.#locale;
    }
    /**
     * Set the locale
     */
    setLocale(locale) {
        this.#locale = locale;
        return this;
    }
    /**
     * The current route
     */
    get route() {
        return this.#route;
    }
    /**
     * Set the current route
     */
    setRoute(route) {
        this.#route = route;
        return this;
    }
    /**
     * Get the client manifest as JSON
     */
    getClientManifestAsJSON() {
        try {
            const serializedManifest = superjson_1.default.serialize(this.#clientManifest);
            return JSON.stringify(serializedManifest, null, radonis_shared_1.isProduction ? 0 : 2);
        }
        catch {
            throw new exceptions_1.E_CANNOT_SERIALIZE_MANIFEST();
        }
    }
    /**
     * Set the server manifest on the global scope
     */
    setServerManifestOnGlobalScope() {
        globalThis.radonisManifest = this.#serverManifest;
    }
    /**
     * Reset for a new request
     */
    reset() {
        this.#setDefaults();
    }
    /**
     * Set the defaults
     */
    #setDefaults() {
        this.#hydration = {};
        this.#globals = {};
        this.#locale = constants_1.DEFAULT_LOCALE;
        this.#route = null;
    }
}
exports.ManifestManager = ManifestManager;
//# sourceMappingURL=main.js.map