"use strict";
/*
 * @microeinhundert/radonis-server
 *
 * (c) Leon Seipp <l.seipp@microeinhundert.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.PluginsManager = void 0;
const exceptions_1 = require("../exceptions");
/**
 * Service for managing plugins
 */
class PluginsManager {
    /**
     * The singleton instance
     */
    static instance;
    /**
     * Get the singleton instance
     */
    static getSingletonInstance(...args) {
        return (PluginsManager.instance = PluginsManager.instance ?? new PluginsManager(...args));
    }
    /**
     * The installed plugins
     */
    #installedPlugins;
    /**
     * The registered client hooks
     */
    onInitClientHooks;
    beforeHydrateHooks;
    /**
     * The registered server hooks
     */
    onBootServerHooks;
    beforeRequestHooks;
    afterRequestHooks;
    beforeRenderHooks;
    afterRenderHooks;
    constructor() {
        this.#setDefaults();
    }
    /**
     * Install one or multiple plugins
     */
    install(targetEnvironment, ...plugins) {
        for (const plugin of plugins) {
            this.#installOrFail(targetEnvironment, plugin);
            this.#checkForConflicts(targetEnvironment);
            this.#registerHooks(targetEnvironment, plugin);
        }
    }
    /**
     * Execute hooks of a specific type
     */
    async execute(type, initialBuilderValue, ...args) {
        const hooks = this[`${type}Hooks`];
        let builderValue = initialBuilderValue;
        for (const hook of hooks) {
            const builderOrVoid = await hook.apply(null, args);
            if (typeof builderOrVoid === 'function') {
                builderValue = await builderOrVoid.apply(null, [builderValue]);
            }
        }
        return builderValue;
    }
    /**
     * Install a plugin or fail if it is incompatible
     */
    #installOrFail(targetEnvironment, { name: pluginName, environments, conflictsWith }) {
        if (this.#installedPlugins.has(pluginName)) {
            throw new exceptions_1.E_PLUGIN_ALREADY_INSTALLED([pluginName]);
        }
        if (environments?.length) {
            for (const environment of ['server', 'client']) {
                if (targetEnvironment === environment && !environments.includes(environment)) {
                    throw new exceptions_1.E_PLUGIN_NOT_INSTALLABLE([pluginName, environment]);
                }
            }
        }
        this.#installedPlugins.set(pluginName, { environments, conflictsWith });
    }
    /**
     * Check for conflicts between installed plugins
     */
    #checkForConflicts(targetEnvironment) {
        for (const [pluginName, { environments, conflictsWith }] of this.#installedPlugins) {
            if (!environments?.includes(targetEnvironment)) {
                continue;
            }
            const conflictingPlugins = conflictsWith?.filter((conflictingPlugin) => this.#installedPlugins.has(conflictingPlugin));
            if (conflictingPlugins?.length) {
                throw new exceptions_1.E_CONFLICTING_PLUGINS([pluginName, conflictingPlugins.join(', ')]);
            }
        }
    }
    /**
     * Register the hooks of a plugin
     */
    #registerHooks(targetEnvironment, plugin) {
        switch (targetEnvironment) {
            case 'client': {
                plugin.onInitClient && this.onInitClientHooks.push(plugin.onInitClient);
                plugin.beforeHydrate && this.beforeHydrateHooks.push(plugin.beforeHydrate);
                break;
            }
            case 'server': {
                plugin.onBootServer && this.onBootServerHooks.push(plugin.onBootServer);
                plugin.beforeRequest && this.beforeRequestHooks.push(plugin.beforeRequest);
                plugin.afterRequest && this.afterRequestHooks.push(plugin.afterRequest);
                plugin.beforeRender && this.beforeRenderHooks.push(plugin.beforeRender);
                plugin.afterRender && this.afterRenderHooks.push(plugin.afterRender);
            }
        }
    }
    /**
     * Set the defaults
     */
    #setDefaults() {
        this.#installedPlugins = new Map();
        this.onInitClientHooks = [];
        this.beforeHydrateHooks = [];
        this.onBootServerHooks = [];
        this.beforeRequestHooks = [];
        this.afterRequestHooks = [];
        this.beforeRenderHooks = [];
        this.afterRenderHooks = [];
    }
}
exports.PluginsManager = PluginsManager;
//# sourceMappingURL=main.js.map