"use strict";
/*
 * @microeinhundert/radonis-server
 *
 * (c) Leon Seipp <l.seipp@microeinhundert.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.Renderer = void 0;
const tslib_1 = require("tslib");
const node_stream_1 = require("node:stream");
const radonis_shared_1 = require("@microeinhundert/radonis-shared");
const flattie_1 = require("flattie");
const isbot_1 = tslib_1.__importDefault(require("isbot"));
const minipass_1 = tslib_1.__importDefault(require("minipass"));
const react_1 = require("react");
const react_2 = require("react");
const server_1 = require("react-dom/server");
const default_error_page_1 = require("../components/default_error_page");
const extract_root_routes_1 = require("../utils/extract_root_routes");
const generate_html_stream_1 = require("../utils/generate_html_stream");
const transform_route_node_1 = require("../utils/transform_route_node");
const with_context_providers_1 = require("../utils/with_context_providers");
const constants_1 = require("./constants");
/**
 * Service for rendering
 */
class Renderer {
    /**
     * The singleton instance
     */
    static instance;
    /**
     * Get the singleton instance
     */
    static getSingletonInstance(...args) {
        return (Renderer.instance = Renderer.instance ?? new Renderer(...args));
    }
    /**
     * The Radonis config
     */
    #config;
    /**
     * The application
     */
    #application;
    /**
     * The Radonis services
     */
    #assetsManager;
    #headManager;
    #hydrationManager;
    #manifestManager;
    #pluginsManager;
    /**
     * The Adonis services
     */
    #logger;
    #router;
    #i18nManager;
    /**
     * The context
     */
    #context;
    /**
     * The error pages
     */
    #errorPages;
    /**
     * The flush callbacks
     */
    #flushCallbacks;
    constructor(config, application) {
        this.#config = config;
        this.#application = application;
        this.#assetsManager = application.container.resolveBinding('Microeinhundert/Radonis/AssetsManager');
        this.#headManager = application.container.resolveBinding('Microeinhundert/Radonis/HeadManager');
        this.#hydrationManager = application.container.resolveBinding('Microeinhundert/Radonis/HydrationManager');
        this.#manifestManager = application.container.resolveBinding('Microeinhundert/Radonis/ManifestManager');
        this.#pluginsManager = application.container.resolveBinding('Microeinhundert/Radonis/PluginsManager');
        this.#logger = application.container.resolveBinding('Adonis/Core/Logger');
        this.#router = application.container.resolveBinding('Adonis/Core/Route');
        this.#i18nManager = application.container.resolveBinding('Adonis/Addons/I18n');
        this.#setDefaults();
    }
    /**
     * Get for request
     */
    getForRequest(httpContext) {
        this.reset();
        this.#router.commit();
        /**
         * Set the context
         */
        this.#context = {
            application: this.#application,
            httpContext,
            router: this.#router,
        };
        const locale = this.#extractUserLocale(httpContext);
        /**
         * Initialize the manifest
         */
        this.#manifestManager
            .setLocale(locale)
            .setFlashMessages((0, flattie_1.flattie)(httpContext.session.flashMessages.all()))
            .setMessages(this.#i18nManager.getTranslationsFor(locale))
            .setRoutes((0, extract_root_routes_1.extractRootRoutes)(this.#router))
            .setRoute({
            ...(0, transform_route_node_1.transformRouteNode)(httpContext.route),
            params: httpContext.request.params(),
            searchParams: httpContext.request.qs(),
        });
        return this;
    }
    /**
     * Set the head title for the current request
     */
    withHeadTitle(title) {
        this.#headManager.setTitle(title);
        return this;
    }
    /**
     * Add head meta for the current request
     */
    withHeadMeta(meta) {
        this.#headManager.addMeta(meta);
        return this;
    }
    /**
     * Add head tags for the current request
     */
    withHeadTags(tags) {
        this.#headManager.addTags(tags);
        return this;
    }
    /**
     * Add globals for the current request
     */
    withGlobals(globals) {
        this.#manifestManager.addGlobals(globals);
        return this;
    }
    /**
     * Add error pages for the current request
     */
    withErrorPages(errorPages) {
        this.#errorPages = { ...this.#errorPages, ...errorPages };
        return this;
    }
    /**
     * Add flush callbacks to be executed after
     * rendering the view for the current request
     */
    withFlushCallbacks(flushCallbacks) {
        this.#flushCallbacks = [...this.#flushCallbacks, ...flushCallbacks];
        return this;
    }
    /**
     * Render the view and stream the response
     */
    async render(Component, props, options) {
        const { request } = this.#context.httpContext;
        /**
         * If the request accepts HTML,
         * return the rendered view
         */
        if (request.accepts(['html'])) {
            /**
             * Set the title on the HeadManager
             */
            if (options?.head?.title) {
                this.#headManager.setTitle(options.head.title);
            }
            /**
             * Add meta to the HeadManager
             */
            if (options?.head?.meta) {
                this.#headManager.addMeta(options.head.meta);
            }
            /**
             * Add tags to the HeadManager
             */
            if (options?.head?.tags) {
                this.#headManager.addTags(options.head.tags);
            }
            /**
             * Add globals to the ManifestManager
             */
            if (options?.globals) {
                this.#manifestManager.addGlobals(options.globals);
            }
            /**
             * Set the server manifest on the global scope
             */
            this.#manifestManager.setServerManifestOnGlobalScope();
            try {
                /**
                 * Render view
                 */
                const readable = await this.#renderToReadable(Component, props);
                this.#context.httpContext.response
                    .header('Content-Type', 'text/html')
                    .stream(readable.pipe(this.#createAfterRenderTransform()));
            }
            catch (error) {
                this.#logger.error(error);
                /**
                 * Render error page
                 */
                const readable = await this.#renderToReadable(this.#errorPages[500] ?? default_error_page_1.DefaultErrorPage, { error });
                this.#context.httpContext.response
                    .header('Content-Type', 'text/html')
                    .stream(readable.pipe(this.#createAfterRenderTransform()));
            }
        }
        return props;
    }
    /**
     * Reset for a new request
     */
    reset() {
        this.#setDefaults();
    }
    /**
     * Get the markup containing the head as well as the opening html tag
     */
    #getHeadMarkup() {
        return [
            '<!DOCTYPE html>',
            `<html ${(0, radonis_shared_1.stringifyAttributes)({ lang: this.#manifestManager.locale })}>`,
            this.#headManager.getMarkup(),
        ].join('\n');
    }
    /**
     * Get the stream containing the body
     */
    #getBodyStream(tree) {
        const { request } = this.#context.httpContext;
        const userAgent = request.header('user-agent');
        const isBot = (0, isbot_1.default)(userAgent);
        const duplex = new minipass_1.default();
        return new Promise((resolve, reject) => {
            try {
                const { pipe, abort } = (0, server_1.renderToPipeableStream)((0, react_2.createElement)(react_2.StrictMode, null, (0, react_2.createElement)('body', null, tree)), {
                    [this.#config.server.streaming && !isBot ? 'onShellReady' : 'onAllReady']() {
                        resolve(pipe(duplex));
                    },
                    onShellError: (error) => {
                        reject(error);
                    },
                    onError: (error) => {
                        reject(error);
                    },
                });
                setTimeout(abort, constants_1.ABORT_DELAY);
            }
            catch (error) {
                reject(error);
            }
        });
    }
    /**
     * Get the markup containing the footer scripts as well as the closing html tag
     */
    async #getFooterMarkup() {
        const resolvedFlushCallbacks = await Promise.all(this.#flushCallbacks.map((flushCallback) => flushCallback.apply(null, [])));
        const flushCallbackInjects = resolvedFlushCallbacks.filter((value) => !!value);
        const scripts = this.#assetsManager.requiredAssets.map((asset) => {
            this.#hydrationManager.requireTokens(asset.tokens);
            return (0, react_2.createElement)('script', {
                type: 'module',
                defer: true,
                src: asset.path,
            });
        });
        const tree = (0, react_2.createElement)(react_1.Fragment, null, (0, react_2.createElement)('script', {
            id: 'rad-manifest',
            dangerouslySetInnerHTML: {
                __html: `window.radonisManifest = ${this.#manifestManager.getClientManifestAsJSON()}`,
            },
        }), ...flushCallbackInjects, ...scripts);
        return [(0, server_1.renderToStaticMarkup)(tree), '</html>'].join('\n');
    }
    /**
     * Render to a Readable
     */
    async #renderToReadable(Component, props) {
        const tree = (0, with_context_providers_1.withContextProviders)(this, this.#assetsManager, this.#manifestManager, this.#context, 
        /* @ts-ignore Unsure why this errors */
        await this.#pluginsManager.execute('beforeRender', (0, react_2.createElement)(Component, props), {
            ctx: this.#context.httpContext,
            manifest: this.#manifestManager,
            props: props,
        }));
        const bodyStream = await this.#getBodyStream(tree);
        const htmlStream = (0, generate_html_stream_1.generateHtmlStream)({
            head: () => this.#getHeadMarkup(),
            body: () => bodyStream,
            footer: () => this.#getFooterMarkup(),
        });
        return node_stream_1.Readable.from(htmlStream);
    }
    /**
     * Create the Transform calling the `afterRender`
     * plugin hook for every chunk of rendered HTML
     */
    #createAfterRenderTransform() {
        return new node_stream_1.Transform({
            transform: async (chunk, _, callback) => {
                callback(null, await this.#pluginsManager.execute('afterRender', chunk.toString(), { ctx: this.#context.httpContext }));
            },
        });
    }
    /**
     * Extract the user locale from the HttpContext
     */
    #extractUserLocale({ request }) {
        const supportedLocales = this.#i18nManager.supportedLocales();
        return request.language(supportedLocales) || request.input('lang') || this.#i18nManager.defaultLocale;
    }
    /**
     * Set the defaults
     */
    #setDefaults() {
        this.#context = null;
        this.#errorPages = {};
        this.#flushCallbacks = [];
    }
}
exports.Renderer = Renderer;
//# sourceMappingURL=main.js.map